@@ -33,10 +33,9 @@ OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMA
 #include "CAMsg.hpp"
 #include "CACmdLnOptions.hpp"
 #include "CASocketAddrUnix.hpp"
-#include "CASocketAddrINet.hpp"
-#include "CAUtil.hpp"
-#include "CAInfoService.hpp"
 #include "CACertStore.hpp"
+#include "CABase64.hpp"
+#include "CAPool.hpp"
 #include "xml/DOM_Output.hpp"
 
 extern CACmdLnOptions options;
@@ -81,41 +80,35 @@ SINT32 CALastMix::initOnce()
 
 SINT32 CALastMix::init()
 	{
-		if(mRSA.generateKeyPair(1024)!=E_SUCCESS)
+		m_pRSA=new CAASymCipher();	
+		if(m_pRSA->generateKeyPair(1024)!=E_SUCCESS)
 			{
 				CAMsg::printMsg(LOG_CRIT,"Could not generate a valid key pair\n");
 				return E_UNKNOWN;
 			}
+		
+		if(m_pSignature!=NULL&&options.isInfoServiceEnabled())
+			{
+				m_pInfoService=new CAInfoService();
+				m_pInfoService->setSignature(m_pSignature);
+				m_pInfoService->start();
+			}
 
 		CAMsg::printMsg(LOG_INFO,"Waiting for Connection from previous Mix...\n");
-/*		CASocketAddr* pAddrListen;
-		UINT8 path[255];
-		if(options.getServerHost(path,255)==E_SUCCESS&&path[0]=='/') //unix domain
-			{
-#ifdef HAVE_UNIX_DOMAIN_PROTOCOL
-				pAddrListen=new CASocketAddrUnix();
-				((CASocketAddrUnix*)pAddrListen)->setPath((char*)path);
-#else
-				CAMsg::printMsg(LOG_CRIT,"I do not understand the Unix Domain Protocol!\n");
-				return E_UNKNOWN;
-#endif
-			}
-		else
-			{
-				pAddrListen=new CASocketAddrINet();
-				((CASocketAddrINet*)pAddrListen)->setAddr(path,options.getServerPort());
-			}
-			*/
-		ListenerInterface oListener;
-		options.getListenerInterface(oListener,1);
+		CAListenerInterface*  pListener=NULL;
+		pListener=options.getListenerInterface(1);
+		const CASocketAddr* pAddr=NULL;
+		if(pListener!=NULL)
+			pAddr=pListener->getAddr();
+		delete pListener;
 		m_pMuxIn=new CAMuxSocket();
-		if(m_pMuxIn->accept(*oListener.addr)!=E_SUCCESS)
+		SINT32 ret=m_pMuxIn->accept(*pAddr);
+		delete pAddr;
+		if(ret!=E_SUCCESS)
 		    {
-					delete oListener.addr;
 					CAMsg::printMsg(LOG_CRIT," failed!\n");
 					return E_UNKNOWN;
 		    }
-		delete oListener.addr;
 		((CASocket*)*m_pMuxIn)->setRecvBuff(500*MIXPACKET_SIZE);
 		((CASocket*)*m_pMuxIn)->setSendBuff(500*MIXPACKET_SIZE);
 		if(((CASocket*)*m_pMuxIn)->setSendLowWat(MIXPACKET_SIZE)!=E_SUCCESS)
@@ -135,6 +128,16 @@ SINT32 CALastMix::init()
 		return processKeyExchange();
 	}
 
+/** Processes the startup communication with the preceeding mix.
+	* \li Step 1: Mix \e n-1 open a TCP/IP Connection\n
+	* \li Step 2: LastMix sends XML struct describing itself, 
+	*					containing PubKey of LastMix, signed by LastMix 
+	*					(see \ref XMLInterMixInitSendFromLast "XML structs")\n
+	* \li Step 3: Mix \e n-1 sends XML struct containing encrpyted (with PubKey) 
+	*					Symetric Key used for	interlink encryption between 
+	*         Mix \e n-1 <---> LastMix, signed by Mix \e n-1
+	*					(see \ref XMLInterMixInitAnswer "XML structs")\n
+	*/
 SINT32 CALastMix::processKeyExchange()
 	{
 		DOM_Document doc=DOM_Document::createDocument();
@@ -147,13 +150,41 @@ SINT32 CALastMix::processKeyExchange()
 		elemMix.setAttribute("id",(char*)idBuff);
 		elemMixes.appendChild(elemMix);
 
+		//Inserting MixProtocol Version (0.3)
+		DOM_Element elemMixProtocolVersion=doc.createElement("MixProtocolVersion");
+		elemMix.appendChild(elemMixProtocolVersion);
+		setDOMElementValue(elemMixProtocolVersion,(UINT8*)"0.3");
+
+		//Inserting RSA-Key
 		DOM_DocumentFragment tmpDocFrag;
-		mRSA.getPublicKeyAsDocumentFragment(tmpDocFrag);
+		m_pRSA->getPublicKeyAsDocumentFragment(tmpDocFrag);
 		DOM_Node nodeRsaKey=doc.importNode(tmpDocFrag,true);
 		elemMix.appendChild(nodeRsaKey);
 		tmpDocFrag=0;
-
-		m_pSignature->signXML(elemMix);
+		//inserting Nonce
+		DOM_Element elemNonce=doc.createElement("Nonce");
+		UINT8 arNonce[16];
+		getRandom(arNonce,16);
+		UINT8 tmpBuff[50];
+		UINT32 tmpLen=50;
+		CABase64::encode(arNonce,16,tmpBuff,&tmpLen);
+		tmpBuff[tmpLen]=0;
+		setDOMElementValue(elemNonce,tmpBuff);
+		elemMix.appendChild(elemNonce);
+		
+		CACertificate* ownCert=options.getOwnCertificate();
+		if(ownCert==NULL)
+			{
+				CAMsg::printMsg(LOG_DEBUG,"Own Test Cert is NULL -- so it could not be inserted into signed KeyInfo send to users...\n");
+			}	
+		CACertStore* tmpCertStore=new CACertStore();
+		tmpCertStore->add(ownCert);
+		if(m_pSignature->signXML(elemMix,tmpCertStore)!=E_SUCCESS)
+			{
+				CAMsg::printMsg(LOG_DEBUG,"Could not sign KeyInfo send to users...\n");
+			}
+		delete ownCert;
+		delete tmpCertStore;
 		
 		UINT32 len=0;
 		UINT8* messageBuff=DOM_Output::dumpToMem(doc,&len);
@@ -193,18 +224,39 @@ SINT32 CALastMix::processKeyExchange()
 				delete []messageBuff;
 				return E_UNKNOWN;
 			}
+		//Verifying nonce!
+		MemBufInputSource oInput(messageBuff,len,"tmp");
+		DOMParser oParser;
+		oParser.parse(oInput);
+		doc=oParser.getDocument();
+		DOM_Element elemRoot=doc.getDocumentElement();
+		elemNonce=NULL;
+		getDOMChildByName(elemRoot,(UINT8*)"Nonce",elemNonce,false);
+		tmpLen=50;
+		memset(tmpBuff,0,tmpLen);
+		if(elemNonce==NULL||getDOMElementValue(elemNonce,tmpBuff,&tmpLen)!=E_SUCCESS||
+			CABase64::decode(tmpBuff,tmpLen,tmpBuff,&tmpLen)!=E_SUCCESS||
+			tmpLen!=SHA_DIGEST_LENGTH ||
+			memcmp(SHA1(arNonce,16,NULL),tmpBuff,SHA_DIGEST_LENGTH)!=0
+			)
+			{
+				CAMsg::printMsg(LOG_CRIT,"Couldt not verify the Nonce!\n");		
+				delete []messageBuff;
+				return E_UNKNOWN;
+			}
+		
 		CAMsg::printMsg(LOG_INFO,"Verified the symetric key!\n");		
 		
-		UINT8 key[50];
-		UINT32 keySize=50;
-		SINT32 ret=decodeXMLEncryptedKey(key,&keySize,messageBuff,len,&mRSA);
+		UINT8 key[150];
+		UINT32 keySize=150;
+		SINT32 ret=decodeXMLEncryptedKey(key,&keySize,messageBuff,len,m_pRSA);
 		delete []messageBuff;
-		if(ret!=E_SUCCESS)
+		if(ret!=E_SUCCESS||keySize!=64)
 			{
 				CAMsg::printMsg(LOG_CRIT,"Couldt not decrypt the symetric key!\n");		
 				return E_UNKNOWN;
 			}
-		if(m_pMuxIn->setKey(key,keySize)!=E_SUCCESS)
+		if(m_pMuxIn->setReceiveKey(key,32)!=E_SUCCESS||m_pMuxIn->setSendKey(key+32,32))
 			{
 				CAMsg::printMsg(LOG_CRIT,"Couldt not set the symetric key to be used by the MuxSocket!\n");		
 				return E_UNKNOWN;
@@ -221,7 +273,7 @@ SINT32 CALastMix::reconfigure()
 		return E_SUCCESS;
 	}
 
-THREAD_RETURN loopLog(void* param)
+THREAD_RETURN lm_loopLog(void* param)
 	{
 		CALastMix* pLastMix=(CALastMix*)param;
 		pLastMix->m_bRunLog=true;
@@ -247,8 +299,57 @@ THREAD_RETURN loopLog(void* param)
 
 #define _SEND_TIMEOUT (UINT32)5000 //5 Seconds...
 
+/**How to end this thread:
+1. Close connection to next mix
+2. put a byte in the Mix-Output-Queue
+*/
+THREAD_RETURN lm_loopSendToMix(void* param)
+	{
+		CAQueue* pQueue=((CALastMix*)param)->m_pQueueSendToMix;
+		CAMuxSocket* pMuxSocket=((CALastMix*)param)->m_pMuxIn;
+		
+		UINT32 len;
+#ifndef USE_POOL
+		UINT8* buff=new UINT8[0xFFFF];
+		for(;;)
+			{
+				len=MIXPACKET_SIZE;
+				SINT32 ret=pQueue->getOrWait(buff,&len);
+				if(ret!=E_SUCCESS||len!=MIXPACKET_SIZE)
+					break;
+				if(pMuxSocket->send((MIXPACKET*)buff)!=MIXPACKET_SIZE)
+					break;
+			}
+		delete []buff;
+#else
+		CAPool* pPool=new CAPool(MIX_POOL_SIZE);
+		MIXPACKET* pMixPacket=new MIXPACKET;
+		for(;;)
+			{
+				len=MIXPACKET_SIZE;
+				SINT32 ret=pQueue->getOrWait((UINT8*)pMixPacket,&len,MIX_POOL_TIMEOUT);
+				if(ret==E_TIMEDOUT)
+					{
+						pMixPacket->flags=0;
+						pMixPacket->channel=DUMMY_CHANNEL;
+						getRandom(pMixPacket->data,DATA_SIZE);
+					}
+				else if(ret!=E_SUCCESS||len!=MIXPACKET_SIZE)
+					break;
+				pPool->pool(pMixPacket);
+				if(pMuxSocket->send(pMixPacket)!=MIXPACKET_SIZE)
+					break;
+			}
+		delete pMixPacket;
+		delete pPool;
+#endif
+		CAMsg::printMsg(LOG_DEBUG,"Exiting Thread SendToMix\n");
+		THREAD_RETURN_SUCCESS;
+	}
+
 SINT32 CALastMix::loop()
 	{
+#ifndef NEW_MIX_TYPE
 		//CASocketList  oSocketList;
 		CALastMixChannelList* pChannelList=new CALastMixChannelList;
 		CASocketGroup osocketgroupCacheRead;
@@ -261,32 +362,30 @@ SINT32 CALastMix::loop()
 		UINT8 rsaBuff[RSA_SIZE];
 		//CONNECTION* tmpCon;
 //		HCHANNEL tmpID;
-		CAQueue oqueueMixIn;
+		m_pQueueSendToMix=new CAQueue();
 		UINT8* tmpBuff=new UINT8[MIXPACKET_SIZE];
 		osocketgroupMixIn.add(*m_pMuxIn);
 		((CASocket*)*m_pMuxIn)->setNonBlocking(true);
 		m_pMuxIn->setCrypt(true);
 		bool bAktiv;
 		UINT32 countCacheAddresses=m_oCacheLB.getElementCount();
-		CAInfoService* pInfoService=NULL;
-		if(m_pSignature!=NULL&&options.isInfoServiceEnabled())
-			{
-				pInfoService=new CAInfoService();
-				pInfoService->setSignature(m_pSignature);
-				pInfoService->sendHelo();
-				pInfoService->start();
-			}
 		m_logUploadedPackets=m_logDownloadedPackets=0;
 		set64((UINT64&)m_logUploadedBytes,(UINT32)0);
 		set64((UINT64&)m_logDownloadedBytes,(UINT32)0);
 		CAThread oLogThread;
-		oLogThread.setMainLoop(loopLog);
+		oLogThread.setMainLoop(lm_loopLog);
 		oLogThread.start(this);
 
 		#ifdef LOG_CHANNEL
 			UINT64 current_millis;
 			UINT32 diff_time; 
 		#endif
+		#ifdef USE_POOL
+			CAPool* pPool=new CAPool(MIX_POOL_SIZE);
+		#endif
+		CAThread threadSendToMix;
+		threadSendToMix.setMainLoop(lm_loopSendToMix);
+		threadSendToMix.start(this);
 
 		for(;;)
 			{
@@ -310,22 +409,36 @@ SINT32 CALastMix::loop()
 									break;
 								//else one packet received
 								m_logUploadedPackets++;
+								#ifdef USE_POOL
+									pPool->pool(pMixPacket);
+								#endif
 								pChannelListEntry=pChannelList->get(pMixPacket->channel);
 								if(pChannelListEntry==NULL)
 									{
-										if(pMixPacket->flags==CHANNEL_OPEN_OLD||pMixPacket->flags==CHANNEL_OPEN_NEW)
+										if(pMixPacket->flags==CHANNEL_OPEN)
 											{
 												#if defined(_DEBUG1) 
 														CAMsg::printMsg(LOG_DEBUG,"New Connection from previous Mix!\n");
 												#endif
-				
+												
+												m_pRSA->decrypt(pMixPacket->data,rsaBuff);
+												#ifdef WITH_TIMESTAMP
+													//CAMsg::printMsg(LOG_DEBUG,"Timestamp is: %X %X\n", *(rsaBuff+KEY_SIZE), *(rsaBuff+KEY_SIZE+1));
+												#endif
+												#ifdef REPLAY_DETECTION
+													if(!validTimestampAndFingerprint(rsaBuff, KEY_SIZE, (rsaBuff+KEY_SIZE)))
+														{
+															CAMsg::printMsg(LOG_INFO,"Duplicate packet ignored.\n");
+															continue;
+														}
+												#endif
 												CASymCipher* newCipher=new CASymCipher();
-												mRSA.decrypt(pMixPacket->data,rsaBuff);
 												newCipher->setKeyAES(rsaBuff);
 												newCipher->decryptAES(pMixPacket->data+RSA_SIZE,
-																							pMixPacket->data+RSA_SIZE-KEY_SIZE,
+																							pMixPacket->data+RSA_SIZE-(KEY_SIZE+TIMESTAMP_SIZE),
 																							DATA_SIZE-RSA_SIZE);
-												memcpy(pMixPacket->data,rsaBuff+KEY_SIZE,RSA_SIZE-KEY_SIZE);
+												memcpy(	pMixPacket->data,rsaBuff+KEY_SIZE+TIMESTAMP_SIZE,
+																RSA_SIZE-(KEY_SIZE+TIMESTAMP_SIZE));
 												CASocket* tmpSocket=new CASocket;										
 												int ret;
 												if(pMixPacket->payload.type==MIX_PAYLOAD_SOCKS)
@@ -351,8 +464,9 @@ SINT32 CALastMix::loop()
 															#endif
 															delete tmpSocket;
 															delete newCipher;
-															m_pMuxIn->close(pMixPacket->channel,tmpBuff);
-															oqueueMixIn.add(tmpBuff,MIXPACKET_SIZE);			
+															getRandom(pMixPacket->data,DATA_SIZE);
+															pMixPacket->flags=CHANNEL_CLOSE;
+															m_pQueueSendToMix->add(pMixPacket,MIXPACKET_SIZE);			
 															m_logDownloadedPackets++;	
 													}
 												else
@@ -371,8 +485,8 @@ SINT32 CALastMix::loop()
 																		memset(crimeBuff,0,PAYLOAD_SIZE+1);
 																		memcpy(crimeBuff,pMixPacket->payload.data,payLen);
 																		UINT32 id=m_pMuxIn->sigCrime(pMixPacket->channel,tmpBuff);
-																		oqueueMixIn.add(tmpBuff,MIXPACKET_SIZE);
-																		CAMsg::printMsg(LOG_CRIT,"Crime detected -- ID: %u -- Content: \n%s\n",id,crimeBuff,payLen);
+																		m_pQueueSendToMix->add(tmpBuff,MIXPACKET_SIZE);
+																		CAMsg::printMsg(LOG_SPECIAL,"Crime detected -- ID: %u -- Content: \n%s\n",id,crimeBuff,payLen);
 																	}
 															#endif
 															if(payLen>PAYLOAD_SIZE||tmpSocket->sendTimeOut(pMixPacket->payload.data,payLen,_SEND_TIMEOUT)==SOCKET_ERROR)
@@ -383,8 +497,9 @@ SINT32 CALastMix::loop()
 																	tmpSocket->close();
 																	delete tmpSocket;
 																	delete newCipher;
-																	m_pMuxIn->close(pMixPacket->channel,tmpBuff);
-																	oqueueMixIn.add(tmpBuff,MIXPACKET_SIZE);			
+																	getRandom(pMixPacket->data,DATA_SIZE);
+																	pMixPacket->flags=CHANNEL_CLOSE;
+																	m_pQueueSendToMix->add(pMixPacket,MIXPACKET_SIZE);			
 																	m_logDownloadedPackets++;	
 																}
 															else
@@ -455,8 +570,9 @@ SINT32 CALastMix::loop()
 														delete pChannelListEntry->pCipher;
 														delete pChannelListEntry->pQueueSend;
 														pChannelList->removeChannel(pMixPacket->channel);
-														m_pMuxIn->close(pMixPacket->channel,tmpBuff);
-														oqueueMixIn.add(tmpBuff,MIXPACKET_SIZE);
+														getRandom(pMixPacket->data,DATA_SIZE);
+														pMixPacket->flags=CHANNEL_CLOSE;
+														m_pQueueSendToMix->add(pMixPacket,MIXPACKET_SIZE);
 														m_logDownloadedPackets++;	
 													}
 												else
@@ -505,8 +621,10 @@ SINT32 CALastMix::loop()
 														delete pChannelListEntry->pSocket;
 														delete pChannelListEntry->pCipher;
 														delete pChannelListEntry->pQueueSend;
-														m_pMuxIn->close(pChannelListEntry->channelIn,tmpBuff);
-														oqueueMixIn.add(tmpBuff,MIXPACKET_SIZE);			
+														pMixPacket->flags=CHANNEL_CLOSE;
+														getRandom(pMixPacket->data,DATA_SIZE);
+														pMixPacket->channel=pChannelListEntry->channelIn;
+														m_pQueueSendToMix->add(pMixPacket,MIXPACKET_SIZE);			
 														m_logDownloadedPackets++;	
 														pChannelList->removeChannel(pChannelListEntry->channelIn);											 
 													}
@@ -532,7 +650,7 @@ SINT32 CALastMix::loop()
 								if(osocketgroupCacheRead.isSignaled(*(pChannelListEntry->pSocket)))
 									{
 										countRead--;
-										if(oqueueMixIn.getSize()<MAX_MIXIN_SEND_QUEUE_SIZE
+										if(m_pQueueSendToMix->getSize()<MAX_MIXIN_SEND_QUEUE_SIZE
 												#ifdef DELAY_CHANNELS
 													&&(pChannelListEntry->trafficOutToUser<DELAY_CHANNEL_TRAFFIC||isGreater64(aktTime,pChannelListEntry->timeNextSend))
 												#endif
@@ -551,9 +669,11 @@ SINT32 CALastMix::loop()
 														delete pChannelListEntry->pSocket;
 														delete pChannelListEntry->pCipher;
 														delete pChannelListEntry->pQueueSend;
-														m_pMuxIn->close(pChannelListEntry->channelIn,tmpBuff);
+														pMixPacket->flags=CHANNEL_CLOSE;
+														pMixPacket->channel=pChannelListEntry->channelIn;
+														getRandom(pMixPacket->data,DATA_SIZE);
 														pChannelList->removeChannel(pChannelListEntry->channelIn);
-														oqueueMixIn.add(tmpBuff,MIXPACKET_SIZE);			
+														m_pQueueSendToMix->add(pMixPacket,MIXPACKET_SIZE);			
 														m_logDownloadedPackets++;	
 													}
 												else 
@@ -567,8 +687,7 @@ SINT32 CALastMix::loop()
 														pMixPacket->payload.len=htons((UINT16)ret);
 														pMixPacket->payload.type=0;
 														pChannelListEntry->pCipher->decryptAES2(pMixPacket->data,pMixPacket->data,DATA_SIZE);
-														m_pMuxIn->send(pMixPacket,tmpBuff);
-														oqueueMixIn.add(tmpBuff,MIXPACKET_SIZE);
+														m_pQueueSendToMix->add(pMixPacket,MIXPACKET_SIZE);
 														m_logDownloadedPackets++;
 														#if defined(LOG_CHANNEL)
 															pChannelListEntry->packetsDataOutToUser++;
@@ -590,26 +709,8 @@ SINT32 CALastMix::loop()
 //end Step 3
 
 //Step 4 Writing to previous Mix
-				if(!oqueueMixIn.isEmpty()&&osocketgroupMixIn.select(true,0)==1)
-					{
-						countRead=pChannelList->getSize()+1;
-						while(countRead>0&&!oqueueMixIn.isEmpty())
-							{
-								bAktiv=true;
-								countRead--;
-								UINT32 len=MIXPACKET_SIZE;
-								oqueueMixIn.peek(tmpBuff,&len);
-								ret=((CASocket*)*m_pMuxIn)->send(tmpBuff,len);
-								if(ret>0)
-									{
-										oqueueMixIn.remove((UINT32*)&ret);
-									}
-								else if(ret==E_AGAIN)
-									break;
-								else
-									goto ERR;
-							}
-					}
+// Now in a separate Thread!
+//
 //end step 4
 				if(!bAktiv)
 					msSleep(100);
@@ -620,11 +721,12 @@ SINT32 CALastMix::loop()
 
 ERR:
 		CAMsg::printMsg(LOG_CRIT,"Seams that we are restarting now!!\n");
+		m_pMuxIn->close();
+		UINT8 b;
+		m_pQueueSendToMix->add(&b,1);
+		CAMsg::printMsg(LOG_CRIT,"Wait for LoopSendToMix!\n");
+		threadSendToMix.join(); //will not join if queue is empty (and so wating)!!!
 		m_bRunLog=false;
-		if(pInfoService!=NULL)
-			{
-				delete pInfoService;
-			}
 		pChannelListEntry=pChannelList->getFirstSocket();
 		while(pChannelListEntry!=NULL)
 			{
@@ -637,7 +739,13 @@ ERR:
 		delete pChannelList;
 		delete []tmpBuff;
 		delete pMixPacket;
+		delete m_pQueueSendToMix;
+		m_pQueueSendToMix=NULL;
+		#ifdef USE_POOL
+			delete pPool;
+		#endif
 		oLogThread.join();
+#endif //! NEW_MIX_TYPE
 		return E_UNKNOWN;
 	}
 
@@ -646,7 +754,9 @@ ERR:
 		{ //Lots of TODO!!!!
 			//DNS Lookup may block if Host does not exists!!!!!
 			//so we use regexp....
-			UINT8* startOfUrl=(UINT8*)memchr(payLoad,32,payLen); //search for first space...
+			if(payLen<3)
+				return false;
+			UINT8* startOfUrl=(UINT8*)memchr(payLoad,32,payLen-1); //search for first space...
 			if(startOfUrl==NULL)
 				return false;
 			startOfUrl++;
@@ -699,13 +809,20 @@ SINT32 CALastMix::setTargets()
 
 SINT32 CALastMix::clean()
 	{
+		if(m_pInfoService!=NULL)
+			{
+				delete m_pInfoService;
+			}
+		m_pInfoService=NULL;
 		if(m_pMuxIn!=NULL)
 			{
 				m_pMuxIn->close();
 				delete m_pMuxIn;
 			}
 		m_pMuxIn=NULL;
-		mRSA.destroy();
+		if(m_pRSA!=NULL)
+			delete m_pRSA;
+		m_pRSA=NULL;
 //		oSuspendList.clear();
 		return E_SUCCESS;
 	}
