@@ -35,6 +35,8 @@ OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMA
 #include "CAThread.hpp"
 #include "CAInfoService.hpp"
 #include "CAUtil.hpp"
+#include "CABase64.hpp"
+#include "CAPool.hpp"
 #include "xml/DOM_Output.hpp"
 
 extern CACmdLnOptions options;
@@ -55,15 +57,31 @@ SINT32 CAMiddleMix::initOnce()
 		return E_SUCCESS;
 	}
 
+/** Processes key exchange with Mix \e n+1 and Mix \e n-1.
+	* \li Step 1: Opens TCP/IP-Connection to Mix \e n+1. \n
+	* \li Step 2: Receives info about Mix \e n+1 .. LastMix as XML struct 
+	*         (see \ref  XMLInterMixInitSendFromLast "XML struct") \n
+  * \li Step 3: Verfies signature, generates symetric Keys used for link encryption
+	*         with Mix \n+1. \n
+	* \li Step 4: Sends symetric Key to Mix \e n+1, encrypted with PubKey of Mix \e n+1
+	*         (see \ref XMLInterMixInitAnswer "XML struct") \n
+	* \li Step 5: Sends info about Mix \e n .. LastMix as XML struct 
+	*         (see \ref  XMLInterMixInitSendFromLast "XML struct")to Mix \e n-1 \n
+	* \li Step 6: Recevies symetric Key used for link encrpytion with Mix \e n-1 
+	*					(see \ref XMLInterMixInitAnswer "XML struct") \n
+	*
+	* @retval E_SUCCESS if KeyExchange with Mix \e n+1 and Mix \e n-1 was succesful
+	*	@retval E_UNKNOWN otherwise
+	*/
 SINT32 CAMiddleMix::proccessKeyExchange()
 	{
-		UINT8* recvBuff=NULL;
-//		UINT8* infoBuff=NULL;
-		
+		UINT8* recvBuff=NULL;		
 		UINT16 len;
+		SINT32 ret;
+
 		if(((CASocket*)*m_pMuxOut)->receiveFully((UINT8*)&len,2)!=E_SUCCESS)
 			{
-				CAMsg::printMsg(LOG_INFO,"Error receiving Key Info lenght!\n");
+				CAMsg::printMsg(LOG_INFO,"Error receiving Key Info lenght from Mix n+1!\n");
 				return E_UNKNOWN;
 			}
 		len=ntohs(len);
@@ -73,24 +91,30 @@ SINT32 CAMiddleMix::proccessKeyExchange()
 			return E_UNKNOWN;
 		if(((CASocket*)*m_pMuxOut)->receiveFully(recvBuff,len)!=E_SUCCESS)
 			{
+				CAMsg::printMsg(LOG_INFO,"Error receiving Key Info from Mix n+1!\n");
 				delete []recvBuff;
 				return E_UNKNOWN;
 			}
 		recvBuff[len]=0; //make a string
-
 		CAMsg::printMsg(LOG_INFO,"Received Key Info...\n");
-		
-				
 		CAMsg::printMsg(LOG_INFO,"%s\n",recvBuff);
+		
+		//Parsing KeyInfo received from Mix n+1
 		MemBufInputSource oInput(recvBuff,len,"tmpID");
 		DOMParser oParser;
 		oParser.parse(oInput);		
 		DOM_Document doc=oParser.getDocument();
 		delete []recvBuff;
+		if(doc.isNull())
+			{
+				CAMsg::printMsg(LOG_INFO,"Error parsing Key Info from Mix n+1!\n");
+				return E_UNKNOWN;
+			}
 
 		DOM_Element root=doc.getDocumentElement();
 		
-		//Sending symetric key...
+		//Finding first <Mix> entry and sending symetric key...
+		bool bFoundNextMix=false;
 		DOM_Node child=root.getFirstChild();
 		while(child!=NULL)
 			{
@@ -100,37 +124,74 @@ SINT32 CAMiddleMix::proccessKeyExchange()
 						CASignature oSig;
 						CACertificate* nextCert=options.getNextMixTestCertificate();
 						oSig.setVerifyKey(nextCert);
-						SINT32 ret=oSig.verifyXML(child,NULL);
+						ret=oSig.verifyXML(child,NULL);
 						delete nextCert;
 						if(ret!=E_SUCCESS)
-							return E_UNKNOWN;
-
+							{
+								CAMsg::printMsg(LOG_INFO,"Could not verify Key Info from Mix n+1!\n");
+								return E_UNKNOWN;
+							}
+						//extracting Nonce and computing Hash of them
+						DOM_Element elemNonce;
+						getDOMChildByName(child,(UINT8*)"Nonce",elemNonce,false);
+						UINT32 lenNonce=1024;
+						UINT8 arNonce[1024];
+						UINT32 tmpLen=1024;
+						if(elemNonce==NULL)
+							{
+								CAMsg::printMsg(LOG_INFO,"No nonce found in Key Info from Mix n+1!\n");
+								return E_UNKNOWN;
+							}
+						getDOMElementValue(elemNonce,arNonce,&lenNonce);
+						CABase64::decode(arNonce,lenNonce,arNonce,&tmpLen);
+						lenNonce=tmpLen;
+						tmpLen=1024;
+						CABase64::encode(SHA1(arNonce,lenNonce,NULL),SHA_DIGEST_LENGTH,
+																	arNonce,&tmpLen);
+						arNonce[tmpLen]=0;
+						
+						//Extracting PubKey of Mix n+1, generating SymKey for link encryption
+						//with Mix n+1, encrypt and send them
 						DOM_Node rsaKey=child.getFirstChild();
 						CAASymCipher oRSA;
 						oRSA.setPublicKeyAsDOMNode(rsaKey);
-						UINT8 key[16];
-						getRandom(key,16);
-						UINT8 buff[4000];
-						UINT32 bufflen=4000;
-						UINT32 outlen=bufflen+5000;
-						UINT8* out=new UINT8[outlen];
-						encodeXMLEncryptedKey(key,16,buff,&bufflen,&oRSA);
-						m_pSignature->signXML(buff,bufflen,out,&outlen);
-						m_pMuxOut->setKey(key,16);
-						UINT16 size=htons(outlen);
+						UINT8 key[64];
+						getRandom(key,64);
+						DOM_DocumentFragment docfragSymKey;
+						encodeXMLEncryptedKey(key,64,docfragSymKey,&oRSA);
+						DOM_Document docSymKey=DOM_Document::createDocument();
+						docSymKey.appendChild(docSymKey.importNode(docfragSymKey,true));
+						DOM_Element elemRoot=docSymKey.getDocumentElement();
+						DOM_Element elemNonceHash=docSymKey.createElement("Nonce");
+						setDOMElementValue(elemNonceHash,arNonce);						
+						elemRoot.appendChild(elemNonceHash);
+						m_pSignature->signXML(elemRoot);
+						m_pMuxOut->setSendKey(key,32);
+						m_pMuxOut->setReceiveKey(key+32,32);
+						UINT32 outlen=0;
+						UINT8* out=DOM_Output::dumpToMem(docSymKey,&outlen);
+						UINT16 size=htons((UINT16)outlen);
 						((CASocket*)m_pMuxOut)->send((UINT8*)&size,2);
 						((CASocket*)m_pMuxOut)->send(out,outlen);
 						delete[] out;
+						bFoundNextMix=true;
 						break;
 					}
 				child=child.getNextSibling();
 			}
-		
-
+		if(!bFoundNextMix)
+			{
+				CAMsg::printMsg(LOG_INFO,"Error -- no Key Info from Mix n+1 found!\n");
+				return E_UNKNOWN;
+			}
+		// -----------------------------------------
+		// ---- Start exchange with Mix n-1 --------
+		// -----------------------------------------		
+		//Inserting own (key) info
 		DOMString tmpDOMStr=root.getAttribute("count");
 		char* tmpStr=tmpDOMStr.transcode();
 		if(tmpStr==NULL)
-			{//TODO all deletes...
+			{
 				return E_UNKNOWN;
 			}
 		UINT32 count=atol(tmpStr);
@@ -142,69 +203,119 @@ SINT32 CAMiddleMix::proccessKeyExchange()
 		UINT8 tmpBuff[50];
 		options.getMixId(tmpBuff,50); //the mix id...
 		mixNode.setAttribute("id",DOMString((char*)tmpBuff));
+		//Supported Mix Protocol -->currently "0.3"
+		DOM_Element elemMixProtocolVersion=doc.createElement("MixProtocolVersion");
+		mixNode.appendChild(elemMixProtocolVersion);
+		setDOMElementValue(elemMixProtocolVersion,(UINT8*)"0.3");
 
 		DOM_DocumentFragment pDocFragment;
 		m_pRSA->getPublicKeyAsDocumentFragment(pDocFragment); //the key
 		mixNode.appendChild(doc.importNode(pDocFragment,true));
 		pDocFragment=0;
+		//inserting Nonce
+		DOM_Element elemNonce=doc.createElement("Nonce");
+		UINT8 arNonce[16];
+		getRandom(arNonce,16);
+		UINT32 tmpLen=50;
+		CABase64::encode(arNonce,16,tmpBuff,&tmpLen);
+		tmpBuff[tmpLen]=0;
+		setDOMElementValue(elemNonce,tmpBuff);
+		mixNode.appendChild(elemNonce);
 		
-		m_pSignature->signXML(mixNode);
+		CACertificate* ownCert=options.getOwnCertificate();
+		if(ownCert==NULL)
+			{
+				CAMsg::printMsg(LOG_DEBUG,"Own Test Cert is NULL -- so it could not be inserted into signed KeyInfo send to users...\n");
+			}	
+		CACertStore* tmpCertStore=new CACertStore();
+		tmpCertStore->add(ownCert);
+		if(m_pSignature->signXML(mixNode,tmpCertStore)!=E_SUCCESS)
+			{
+				CAMsg::printMsg(LOG_DEBUG,"Could not sign KeyInfo send to users...\n");
+			}
+		delete ownCert;
+		delete tmpCertStore;
 		
 		root.insertBefore(mixNode,root.getFirstChild());
-//		root.appendChild(mixNode);
-//		delete mixNode;
 
-		recvBuff=new UINT8[2048];
-		UINT32 outlen=2048;
-		DOM_Output::dumpToMem(doc,recvBuff+2,&outlen);
+		UINT8* out=new UINT8[0xFFFF];
+		UINT32 outlen=0xFFFD;
+		DOM_Output::dumpToMem(doc,out+2,&outlen);
 		#ifdef _DEBUG
 			CAMsg::printMsg(LOG_DEBUG,"New Key Info size: %u\n",outlen);
 		#endif
-		len=htons(outlen);
-		memcpy(recvBuff,&len,2);
-		if(((CASocket*)*m_pMuxIn)->send(recvBuff,outlen+2)==-1)
+		len=htons((UINT16)outlen);
+		memcpy(out,&len,2);
+		ret=((CASocket*)*m_pMuxIn)->send(out,outlen+2);
+		delete[] out;
+		if(ret<0||(UINT32)ret!=outlen+2)
 			{
 				CAMsg::printMsg(LOG_DEBUG,"Error sending new New Key Info\n");
 				return E_UNKNOWN;
 			}
-		else
-			CAMsg::printMsg(LOG_DEBUG,"Sending new New Key Info succeded\n");
+		CAMsg::printMsg(LOG_DEBUG,"Sending new New Key Info succeded\n");
 		
-			//Now receiving the symmetric key
+			//Now receiving the symmetric key form Mix n-1
 		((CASocket*)*m_pMuxIn)->receive((UINT8*)&len,2);
 		len=ntohs(len);
+		recvBuff=new UINT8[len+1]; //for \0 at the end
 		if(((CASocket*)*m_pMuxIn)->receive(recvBuff,len)!=len)
 			{
-				CAMsg::printMsg(LOG_ERR,"Error receiving symetric key!\n");
+				CAMsg::printMsg(LOG_ERR,"Error receiving symetric key from Mix n-1!\n");
 				delete []recvBuff;
 				return E_UNKNOWN;
 			}
 		recvBuff[len]=0;
 		CAMsg::printMsg(LOG_INFO,"Symmetric Key Info received is:\n");
 		CAMsg::printMsg(LOG_INFO,"%s\n",(char*)recvBuff);		
+		//Parsing doc received
+		MemBufInputSource oInput1(recvBuff,len,"tmp");
+		oParser.parse(oInput1);
+		doc=oParser.getDocument();
+		delete[] recvBuff;
+		if(doc.isNull())
+			{
+				CAMsg::printMsg(LOG_INFO,"Error parsing Key Info from Mix n-1!\n");
+				return E_UNKNOWN;
+			}
+		DOM_Element elemRoot=doc.getDocumentElement();
 		//verify signature
 		CASignature oSig;
 		CACertificate* pCert=options.getPrevMixTestCertificate();
 		oSig.setVerifyKey(pCert);
 		delete pCert;
-		if(oSig.verifyXML(recvBuff,len)!=E_SUCCESS)
+		if(oSig.verifyXML(elemRoot)!=E_SUCCESS)
 			{
-				CAMsg::printMsg(LOG_CRIT,"Couldt not verify the symetric key!\n");		
-				delete []recvBuff;
+				CAMsg::printMsg(LOG_CRIT,"Couldt not verify the symetric key form Mix n-1!\n");		
+				return E_UNKNOWN;
+			}
+		//Verifying nonce
+		elemNonce=NULL;
+		getDOMChildByName(elemRoot,(UINT8*)"Nonce",elemNonce,false);
+		tmpLen=50;
+		memset(tmpBuff,0,tmpLen);
+		if(elemNonce==NULL||getDOMElementValue(elemNonce,tmpBuff,&tmpLen)!=E_SUCCESS||
+			CABase64::decode(tmpBuff,tmpLen,tmpBuff,&tmpLen)!=E_SUCCESS||
+			tmpLen!=SHA_DIGEST_LENGTH ||
+			memcmp(SHA1(arNonce,16,NULL),tmpBuff,SHA_DIGEST_LENGTH)!=0
+			)
+			{
+				CAMsg::printMsg(LOG_CRIT,"Couldt not verify the Nonce!\n");		
 				return E_UNKNOWN;
 			}
 		CAMsg::printMsg(LOG_INFO,"Verified the symetric key!\n");		
 
-		UINT8 key[50];
-		UINT32 keySize=50;
-		decodeXMLEncryptedKey(key,&keySize,recvBuff,len,m_pRSA);
-		if(m_pMuxIn->setKey(key,keySize)!=E_SUCCESS)
+		UINT8 key[150];
+		UINT32 keySize=150;
+	
+		ret=decodeXMLEncryptedKey(key,&keySize,elemRoot,m_pRSA);
+		if(ret!=E_SUCCESS||keySize!=64)
 			{
 				CAMsg::printMsg(LOG_CRIT,"Couldt not set the symetric key to be used by the MuxSocket!\n");		
-				delete []recvBuff;
 				return E_UNKNOWN;
 			}
-		delete [] recvBuff;
+		m_pMuxIn->setReceiveKey(key,32);
+		m_pMuxIn->setSendKey(key+32,32);
 		return E_SUCCESS;
 	}
 
@@ -245,6 +356,14 @@ SINT32 CAMiddleMix::init()
 			}
 		((CASocket*)*m_pMuxOut)->setRecvBuff(50*MIXPACKET_SIZE);
 		((CASocket*)*m_pMuxOut)->setSendBuff(50*MIXPACKET_SIZE);
+
+//We now tell the worl that we are waiting...
+		if(m_pSignature!=NULL&&options.isInfoServiceEnabled())
+			{
+				m_pInfoService=new CAInfoService();
+				m_pInfoService->setSignature(m_pSignature);
+				m_pInfoService->start();
+			}
 #define RETRIES 100
 #define RETRYTIME 30
 		CAMsg::printMsg(LOG_INFO,"Init: Try to connect to next Mix...\n");
@@ -262,10 +381,16 @@ SINT32 CAMiddleMix::init()
 					CAMsg::printMsg(LOG_INFO,"Socket option KEEP-ALIVE returned an error - so also not set!\n");
 			}
 		
-		ListenerInterface oListener;
-		options.getListenerInterface(oListener,1);
+		CAListenerInterface* pListener=NULL;
+		const CASocketAddr* pAddr=NULL;
+		pListener=options.getListenerInterface(1);
+		if(pListener!=NULL)
+			pAddr=pListener->getAddr();
+		delete pListener;
 		m_pMuxIn=new CAMuxSocket();
-		if(m_pMuxIn->accept(*oListener.addr)!=E_SUCCESS)
+		SINT32 ret=m_pMuxIn->accept(*pAddr);
+		delete pAddr;
+		if(ret!=E_SUCCESS)
 			{
 				CAMsg::printMsg(LOG_CRIT,"Error waiting for previous Mix... -- Exiting!\n");				
 				return E_UNKNOWN;
@@ -280,7 +405,12 @@ SINT32 CAMiddleMix::init()
 			}
 		
 
-		proccessKeyExchange();
+		if(proccessKeyExchange()!=E_SUCCESS)
+			{
+				CAMsg::printMsg(LOG_CRIT,"Error in proccessKeyExchange()!\n");
+				return E_UNKNOWN;
+			}
+	
 
 		m_pMiddleMixChannelList=new CAMiddleMixChannelList();
 		
@@ -288,8 +418,9 @@ SINT32 CAMiddleMix::init()
 	}
 
 	
-THREAD_RETURN loopDownStream(void *p)
+THREAD_RETURN mm_loopDownStream(void *p)
 	{
+#ifndef NEW_MIX_TYPE
 		CAMiddleMix* pMix=(CAMiddleMix*)p;
 		HCHANNEL channelIn;
 		CASymCipher* pCipher;
@@ -297,13 +428,31 @@ THREAD_RETURN loopDownStream(void *p)
 		SINT32 ret;
 		CASingleSocketGroup oSocketGroup;
 		oSocketGroup.add(*(pMix->m_pMuxOut));
+#ifdef USE_POOL		
+		CAPool* pPool=new CAPool(MIX_POOL_SIZE);
+#endif
 		for(;;)
 			{
-				ret=oSocketGroup.select(false,1000);
+				#ifndef USE_POOL			
+					ret=oSocketGroup.select(false,1000);
+				#else
+					ret=oSocketGroup.select(false,MIX_POOL_TIMEOUT);
+				#endif
 				if(ret!=1)
 					{
 						if(ret==E_TIMEDOUT)
-							continue;
+							{
+								#ifndef USE_POOL
+									continue;
+								#else
+									pMixPacket->channel=DUMMY_CHANNEL;
+									pMixPacket->flags=CHANNEL_CLOSE;
+									getRandom(pMixPacket->data,DATA_SIZE);
+									pPool->pool(pMixPacket);
+									if(pMix->m_pMuxIn->send(pMixPacket)==SOCKET_ERROR)
+										goto ERR;								
+								#endif
+							}
 						else
 							{
 								CAMsg::printMsg(LOG_CRIT,"loopDownStream -- Fehler bei select() -- goto ERR!\n");
@@ -323,46 +472,51 @@ THREAD_RETURN loopDownStream(void *p)
 								CAMsg::printMsg(LOG_INFO,"loopDownStream received a packet with invalid flags: %0X .  Removing them.\n",(pMixPacket->flags & ~CHANNEL_ALLOWED_FLAGS));
 								pMixPacket->flags&=CHANNEL_ALLOWED_FLAGS;
 							}
-						
+						#ifdef USE_POOL	
+							if(pMixPacket->channel==DUMMY_CHANNEL)
+								{
+									pMixPacket->flags=CHANNEL_CLOSE;
+									getRandom(pMixPacket->data,DATA_SIZE);
+									pPool->pool(pMixPacket);
+									if(pMix->m_pMuxIn->send(pMixPacket)==SOCKET_ERROR)
+										goto ERR;								
+								}
+							else
+						#endif
 						if(pMix->m_pMiddleMixChannelList->getOutToIn(&channelIn,pMixPacket->channel,&pCipher)==E_SUCCESS)
 							{//connection found
-								if(pMixPacket->flags!=CHANNEL_CLOSE)
-									{
-										pMixPacket->channel=channelIn;
-										pCipher->decryptAES2(pMixPacket->data,pMixPacket->data,DATA_SIZE);
-										pCipher->unlock();
-										if(pMix->m_pMuxIn->send(pMixPacket)==SOCKET_ERROR)
-											goto ERR;
-									}
+								pMixPacket->channel=channelIn;
+								#ifdef LOG_CRIME
+								if((pMixPacket->flags&CHANNEL_SIG_CRIME)==CHANNEL_SIG_CRIME)
+									getRandom(pMixPacket->data,DATA_SIZE);
 								else
-									{//connection should be closed
-										pCipher->unlock();
-										if(pMix->m_pMuxIn->close(channelIn)==SOCKET_ERROR)
-											goto ERR;
-										pMix->m_pMiddleMixChannelList->remove(channelIn);
-									}
+								#endif
+								pCipher->decryptAES2(pMixPacket->data,pMixPacket->data,DATA_SIZE);
+								pCipher->unlock();
+								#ifdef USE_POOL
+									pPool->pool(pMixPacket);
+								#endif
+								if(pMix->m_pMuxIn->send(pMixPacket)==SOCKET_ERROR)
+									goto ERR;
 							}
 					}
 			}
 ERR:
 		CAMsg::printMsg(LOG_CRIT,"loopDownStream -- Exiting!\n");
 		delete pMixPacket;
+		#ifdef USE_POOL
+			delete pPool;
+		#endif
 		pMix->m_pMuxIn->close();
 		pMix->m_pMuxOut->close();
+#endif //!NEW_MIX_TYPE
 		THREAD_RETURN_SUCCESS;		
 	}
 
 
 SINT32 CAMiddleMix::loop()
 	{
-		CAInfoService* pInfoService=NULL;
-		if(m_pSignature!=NULL&&options.isInfoServiceEnabled())
-			{
-				pInfoService=new CAInfoService();
-				pInfoService->setSignature(m_pSignature);
-				pInfoService->sendHelo();
-				pInfoService->start();
-			}
+#ifndef NEW_MIX_TYPE
 		MIXPACKET* pMixPacket=new MIXPACKET;
 		HCHANNEL channelOut;
 		CASymCipher* pCipher;
@@ -372,16 +526,34 @@ SINT32 CAMiddleMix::loop()
 		oSocketGroup.add(*m_pMuxIn);
 		m_pMuxIn->setCrypt(true);
 		m_pMuxOut->setCrypt(true);
+		#ifdef USE_POOL		
+			CAPool* pPool=new CAPool(MIX_POOL_SIZE);
+		#endif
 		CAThread oThread;
-		oThread.setMainLoop(loopDownStream);
+		oThread.setMainLoop(mm_loopDownStream);
 		oThread.start(this);
 		for(;;)
 			{
-				ret=oSocketGroup.select(false,1000);
+				#ifndef USE_POOL			
+					ret=oSocketGroup.select(false,1000);
+				#else
+					ret=oSocketGroup.select(false,MIX_POOL_TIMEOUT);
+				#endif
 				if(ret!=1)
 					{
 						if(ret==E_TIMEDOUT)
-							continue;
+							{
+								#ifndef USE_POOL
+									continue;
+								#else
+									pMixPacket->channel=DUMMY_CHANNEL;
+									pMixPacket->flags=CHANNEL_CLOSE;
+									getRandom(pMixPacket->data,DATA_SIZE);
+									pPool->pool(pMixPacket);
+									if(m_pMuxOut->send(pMixPacket)==SOCKET_ERROR)
+										goto ERR;								
+								#endif
+							}
 						else
 							{
 								CAMsg::printMsg(LOG_CRIT,"loopUpStream -- Fehler bei select() -- goto ERR!\n");
@@ -401,43 +573,57 @@ SINT32 CAMiddleMix::loop()
 								CAMsg::printMsg(LOG_INFO,"loopUpStream received a packet with invalid flags: %0X .  Removing them.\n",(pMixPacket->flags & ~CHANNEL_ALLOWED_FLAGS));
 								pMixPacket->flags&=CHANNEL_ALLOWED_FLAGS;
 							}
+						#ifdef USE_POOL	
+							if(pMixPacket->channel==DUMMY_CHANNEL)
+								{
+									pMixPacket->flags=CHANNEL_CLOSE;
+									getRandom(pMixPacket->data,DATA_SIZE);
+									pPool->pool(pMixPacket);
+									if(m_pMuxOut->send(pMixPacket)==SOCKET_ERROR)
+										goto ERR;								
+								}
+							else
+						#endif
 						if(m_pMiddleMixChannelList->getInToOut(pMixPacket->channel,&channelOut,&pCipher)!=E_SUCCESS)
-							{//new connection
-								if(pMixPacket->flags==CHANNEL_OPEN_OLD||pMixPacket->flags==CHANNEL_OPEN_NEW)
+							{//new connection ?
+								if(pMixPacket->flags==CHANNEL_OPEN)
 									{
 										#ifdef _DEBUG
 												CAMsg::printMsg(LOG_DEBUG,"New Connection from previous Mix!\n");
 										#endif
-										pCipher=new CASymCipher();
 										m_pRSA->decrypt(pMixPacket->data,tmpRSABuff);
+										#ifdef REPLAY_DETECTION
+											if(!validTimestampAndFingerprint(tmpRSABuff, KEY_SIZE, (tmpRSABuff+KEY_SIZE)))
+												{
+													CAMsg::printMsg(LOG_INFO,"Duplicate packet ignored.\n");
+													continue;
+												}
+										#endif
+										pCipher=new CASymCipher();
 										pCipher->setKeyAES(tmpRSABuff);
 										pCipher->decryptAES(pMixPacket->data+RSA_SIZE,
-																				pMixPacket->data+RSA_SIZE-KEY_SIZE,
+																				pMixPacket->data+RSA_SIZE-(KEY_SIZE+TIMESTAMP_SIZE),
 																				DATA_SIZE-RSA_SIZE);
-										memcpy(pMixPacket->data,tmpRSABuff+KEY_SIZE,RSA_SIZE-KEY_SIZE);
+										memcpy(pMixPacket->data,tmpRSABuff+KEY_SIZE+TIMESTAMP_SIZE,RSA_SIZE-(KEY_SIZE+TIMESTAMP_SIZE));
 										m_pMiddleMixChannelList->add(pMixPacket->channel,pCipher,&channelOut);
 										pMixPacket->channel=channelOut;
+										#ifdef USE_POOL
+											pPool->pool(pMixPacket);
+										#endif
 										if(m_pMuxOut->send(pMixPacket)==SOCKET_ERROR)
 											goto ERR;
 									}
 							}
 						else
 							{//established connection
-								if(pMixPacket->flags==CHANNEL_CLOSE)
-									{
-										pCipher->unlock();
-										if(m_pMuxOut->close(channelOut)==SOCKET_ERROR)
-											goto ERR;
-										m_pMiddleMixChannelList->remove(pMixPacket->channel);
-									}
-								else
-									{
-										pMixPacket->channel=channelOut;
-										pCipher->decryptAES(pMixPacket->data,pMixPacket->data,DATA_SIZE);
-										pCipher->unlock();
-										if(m_pMuxOut->send(pMixPacket)==SOCKET_ERROR)
-											goto ERR;
-									}
+									pMixPacket->channel=channelOut;
+									pCipher->decryptAES(pMixPacket->data,pMixPacket->data,DATA_SIZE);
+									pCipher->unlock();
+									#ifdef USE_POOL
+										pPool->pool(pMixPacket);
+									#endif
+									if(m_pMuxOut->send(pMixPacket)==SOCKET_ERROR)
+										goto ERR;
 							}
 					}
 			}
@@ -450,12 +636,17 @@ ERR:
 		CAMsg::printMsg(LOG_CRIT,"Seams that we are restarting now!!\n");
 		delete tmpRSABuff;
 		delete pMixPacket;
-		if(pInfoService!=NULL)
-			delete pInfoService;
+		#ifdef USE_POOL
+			delete pPool;
+		#endif
+#endif //!NEW_MIX_TYPE
 		return E_UNKNOWN;
 	}
 SINT32 CAMiddleMix::clean()
 	{
+		if(m_pInfoService!=NULL)
+			delete m_pInfoService;
+		m_pInfoService=NULL;
 		if(m_pMuxIn!=NULL)
 			{
 				m_pMuxIn->close();
