@@ -58,6 +58,10 @@ OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMA
 #ifdef LOG_CRIME
 #include "tre/regex.h"
 #endif
+#ifdef NEW_MIX_TYPE
+	#include "CAFirstMixNewProtocol.hpp"
+	#include "CALastMixNewProtocol.hpp"
+#endif
 //#include "CAPayment.hpp"
 //#ifdef _WIN32
 //HANDLE hEventThreadEnde;
@@ -109,15 +113,9 @@ void signal_hup(int sig)
 	}
 
 
-//Callbackfunction for looking required by OpenSSL
+///Callbackfunction for looking required by OpenSSL
 void openssl_locking_callback(int mode, int type, char *file, int line)
 	{
-/*#if _DEBUG
-		CAMsg::printMsg(LOG_DEBUG,"OpenSSL-Locking: thread=%4d mode=%s lock=%s %s:%d\n",
-			CRYPTO_thread_id(),
-			(mode&CRYPTO_LOCK)?"l":"u",
-			(type&CRYPTO_READ)?"r":"w",file,line);
-#endif*/
 		if (mode & CRYPTO_LOCK)
 			{
 				pOpenSSLMutexes[type].lock();
@@ -127,6 +125,48 @@ void openssl_locking_callback(int mode, int type, char *file, int line)
 				pOpenSSLMutexes[type].unlock();
 			}
 	}
+	
+///Check what the sizes of base types are as expected -- if not kill the programm
+void checkSizesOfBaseTypes()
+	{
+		if(sizeof(SINT8)!=1)
+			{
+				CAMsg::printMsg(LOG_CRIT,"sizeof(SINT8) != 1 --> maybe a compiler (optimization) problem!\n");
+				exit(-1);
+			}
+		if(sizeof(UINT8)!=1)
+			{
+				CAMsg::printMsg(LOG_CRIT,"sizeof(UINT8) != 1 --> maybe a compiler (optimization) problem!\n");
+				exit(-1);
+			}
+		if(sizeof(SINT16)!=2)
+			{
+				CAMsg::printMsg(LOG_CRIT,"sizeof(SINT16) != 2 --> maybe a compiler (optimization) problem!\n");
+				exit(-1);
+			}
+		if(sizeof(UINT16)!=2)
+			{
+				CAMsg::printMsg(LOG_CRIT,"sizeof(UINT16) != 2 --> maybe a compiler (optimization) problem!\n");
+				exit(-1);
+			}
+		if(sizeof(SINT32)!=4)
+			{
+				CAMsg::printMsg(LOG_CRIT,"sizeof(SINT32) != 4 --> maybe a compiler (optimization) problem!\n");
+				exit(-1);
+			}
+		if(sizeof(UINT32)!=4)
+			{
+				CAMsg::printMsg(LOG_CRIT,"sizeof(UINT32) != 4 --> maybe a compiler (optimization) problem!\n");
+				exit(-1);
+			}
+		#ifdef HAVE_NATIVE_UINT64	
+			if(sizeof(UINT64)!=8)
+				{
+					CAMsg::printMsg(LOG_CRIT,"sizeof(UINT64) != 8 --> maybe a compiler (optimization) problem!\n");
+					exit(-1);
+				}
+		#endif
+	}
 
 /** \mainpage 
  
@@ -134,7 +174,7 @@ void openssl_locking_callback(int mode, int type, char *file, int line)
 
 \subsection Basics
 
-The whole system consist of JAP's, mix-servers, cache proxies and the InfoService. (see Figure 1)
+The whole system consists of JAP's, mix-servers, cache proxies and the InfoService. (see Figure 1)
 
 \image html JAPArchitecture.gif "Figure 1: The Architecture of the Anonymous-Service"
 	
@@ -146,38 +186,43 @@ to one or two other mixes. If a mix receives data packets from JAP's and sends t
  mix has only one connection to an other mix. Each mix with two connections to other mixes is called
  a \em middle mix. This type of mix will receive packets form one mix and forwards them to the other mix.
 
-If mixes are connected in a meanigful way a chain is setup up, so what packets are transmitted from JAP's 
+If mixes are connected in a meaningful way a chain is setup up, so what packets are transmitted from JAP's 
 to the cache-proxies and than to the Internet.
-A chain of mixes is called a \em mix-cascade (or \em cascade for short).
+A chain of mixes is called a \em MixCascade (or \em cascade for short).
 Many different cascades could exist at the same time, but JAP can select one and only one at the same time.
 
 Also a mix could only be part of one and only one cascade. If a mix is not part of a cascade, we call it a \em free mix.
 Free mixes are not useable for JAP, but could be connected to build a new cascade.
   
+Beside JAPs and Mixes we have added a third component to our system: the \em InfoService. 
+The InfoService is more or less a database which stores information about 
+the system like available MixCascades, status of Mixes etc.
 \subsection docMux Mulitplexing and Demultiplexing
 
 JAP acts as a local proxy for the browser. 
 The browser opens many connections to the JAP (usually one per HTTP-Request).
 All this connections are multiplexed over the one connection to the first mix JAP is connected to.
-Every connection from the browser is called a \em channel (mix channel or anonymous channel). 
-A first mix sends the stream of packets (from multiple channels) from multiple users to the next mix.
-All over one TCP/IP connection. So JAP and the mixes have to multiplex/demultiplex the channels.
-A channel can transport only fixed sized packets. These packets are called \em mix-packets. 
+Every connection from the browser is called a \em channel (mix channel or \em AnonChannel). 
+A first mix sends the stream of packets (from multiple channels) from multiple users to the next mix (all over one TCP/IP connection). So JAP and the mixes have to multiplex/demultiplex the channels.
+A channel can transport only fixed sized packets. These packets are called \em MixPackets. 
 So a first mix for instance gets many packets from different users in parallel 
 and sends them to the next mix in a serialized way. (see Figure 2)
 
-\image html JAPMixPacketMux.gif "Figure 2: Mux/DeMux of fixed sized mix-packets"
+\image html JAPMixPacketMux.gif "Figure 2: Mux/DeMux of fixed sized MixPackets"
 
 
-Each mix-packet has a size of #MIXPACKET_SIZE (998) bytes. 
+Each MixPacket has a size of #MIXPACKET_SIZE (998) bytes (see Figure 3). 
 The header of each packet is as follows (see also: #MIXPACKET):
-\li 4 bytes \c channel-id
+\li 4 bytes \c channel-ID
 \li 2 bytes \c flags
-\li #DATA_SIZE (992) bytes  \c content (transported bytes)
+\li #DATA_SIZE (992) bytes  \c data (transported bytes)
 
-The content bytes have different meanings in different situations and mixes.
+\image html MixPacketGeneral.gif "Figure 3: General format of a MixPacket
+
+
+The data bytes have different meanings in different situations and mixes.
 For instance the content itself is meaningful only at the last mix (because of multiple encryptions)
-The format of the content (at the last mix) is:
+The format of the data (at the last mix) is:
 \li 2 bytes \c len
 \li 1 byte \c type
 \li #PAYLOAD_SIZE (989) bytes \c payload
@@ -185,30 +230,145 @@ The format of the content (at the last mix) is:
 They payload are the bytes what should be transported from the browser to the Internet 
 or back via the anonymous channel.
 
-The channel-id is changed in every mix. Also the content bytes changes in every mix, 
+The channel-ID changes in every mix. Also the content bytes changes in every mix, 
 because each mix will perform a single encryption/decryption.
 
 \subsection docInterMixEncryption Inter-Mix Encryption
-The stream of mix-packets between to mixes is encrypted using AES-128/128 in OFB-128 mode. Exactly only the
-first part of each mix-packet is encrpyted. (see Figure 3)
+The stream of MixPackets between to mixes is encrypted using AES-128/128 in OFB-128 mode. Exactly only the
+first part (16 bytes) of each MixPacket is encrypted. (see Figure 4)
 
-\image html JAPInterMixEncryption.gif "Figure 3: Encryption between two mixes"
+\image html JAPInterMixEncryption.gif "Figure 4: Encryption between two mixes"
 	
-The encryption is done, so that an attacker could not see the channel-id and flags of a mix-packet. OFB is chosen, so that
-an attacker could not replay a mix-packed. If he replays a mix-packed, than at least the channel-id 
-changes after decrypting. If this mix-packed was the first packed of a channel, than the cryptographic keys
-for this channel will change to (because of the content format for the first packed of a channel, explained later). 
+The encryption is done, so that an attacker could not see the channel-ID and flags of a MixPacket. OFB is chosen, so that
+an attacker could not replay a MixPacket. If he replays a MixPacket, than at least the channel-ID 
+changes after decrypting. If this MixPacket was the first packet of a channel, than the cryptographic keys
+for this channel will change to (because of the content format for the first packet of a channel, explained later). 
+
+For Upstream and Downstream different keys are used. 
+See \ref docCascadeInit "[Cascade setup]" for information about exchange of these keys.
+
+\subsection docChannelSetup AnonChannel establishment
+Before data could be sent through an AnonChannel one has to be established. 
+This is done by sending ChannelOpen messages through the mixes.
+Figure 5 shows the format of such a message how it would arrive for example 
+at the last mix.
+
+\image html MixPacketChannelOpen.gif "Figure 5: ChannelOpen packet (example for the last mix)"
+
+\section docCascadeInit Initialisation of a Cascade
+
+If a Mix startsup it reads its configuration information from a file. See \ref pageMixConfig "[MixConfig]" for more information.
+
+\image html JAPCascadeInit.gif "Figure 6: Steps during Cascade initialisation (3 Mix example)
+
+Figure 6 shows the steps which take place, than a Cascade starts. These steps are illustrated using a 3 Mix example.
+The procedure is as follows:
+
+\li Step 1: Each Mix starts, reads its configuration file (including own key
+ for digital signature and test keys from previous and next mix) and generates a 
+ public key pair of the asymmetric crypto system used by the mix for encryption 
+ (currently this is RSA).
+
+\li Step 2: Mix 2 establishes a TCP/IP connection with Mix 3. 
+No data is transmitted during this step.
+
+\li Step 3: Mix 3 sends its public key  to Mix 2 (signed with its signature key).
+See \ref XMLInterMixInitSendFromLast "[XML]" for a description of the XML struct send in this step. 
+
+\li Step 4: Mix 2 generates and sends a symmetric key to Mix 3, encrypted with the public key 
+of Mix 3, signed by Mix 2. 
+This key is used for inter-mix encryption and is actually a 64 byte octet stream.
+See \ref XMLInterMixInitAnswer "[XML]" for a description of the XML struct send in this step.\n  
+The first 16 bytes a used as a key for 
+inter-mix encryption of packets send from Mix \e n-1 to Mix \e n (e.g. upstream direction).
+The next 16 byte are used as IV for this cipher. The next 16 bytes are used as
+key for downstream direction and the last 16 bytes are the IV for this cipher.
+
+\li Step 5,6,7: This steps are equal to step 2,3,4. Mix 1 establishes a TCP/IP-connection
+with Mix 2. Mix 2 send it publick key to Mix 1 and Mix 1 generates and sends 
+a symmetric key to Mix 2.
+
+\section docMixJap Communication between Mix and JAP
+
+\image html JAPMixJap.gif "Figure 7: Communication between Mix and JAP"
+
+-# JAP opens a TCP/IP connection to a FirstMix
+-# FirstMix sends information about the cascade (including public keys of the Mixes) to the JAP.
+See \ref XMLMixKeyInfo "[XML]" for a description of the XML struct send. 
+-# JAP sends a special MixPacket, containing only 2 symmetric keys and IVs encrypted with the
+public key of the FirstMix. This keys are used for link encryption between JAP and FirstMix.
+Note: At the moment this is binary - but will use XML in the future.
+The data part of this MixPacket is as follows:
+	- the ASCII string: "KEYPACKET"
+	- 16 random bytes used as symmetric key for packets send from JAP to Mix 
+	- 16 random bytes used as IV for this symetric cipher
+	- 16 random bytes used as symetric key for packets send from Mix to JAP 
+	- 16 random bytes used as IV for this symmetric cipher
+	.
+
+-# Normal MixPacket exchange according to the mix protocol.
+
+\section docInfoService Communication with the InfoService
+HTTP GET/POST-requests are used for communication with the InfoService. 
+The filename part of the URL specifies the command (action) to be executed and
+additional information is transfered as XML struct in the body of the HTTP message.
+Example:
+
+\verbatim
+POST /status HTTP/1.0
+Content-Type: text/xml
+Content-Length: 251
+
+<?xml version="1.0" encoding="utf-8" ?> 
+<MixCascadeStatus id="testmix" 
+                  mixedPackets="567899"
+                  nrOfActiveUsers="235"
+                  trafficSituation="78"
+                  LastUpdate="126940258075">
+</MixCascadeStatus>
+\endverbatim
+
+\subsection docMixInfoService Communication between Mix and InfoService
+
+\image html JAPMixInfoService.gif "Figure 8: Communication between Mix and InfoService"
+
+\li 1. HELO-Messages send from each mix to the InfoService every 10 minutes to announce itself.  
+This is a POST request and the command name is: "helo".
+See \ref XMLMixHELO "[XML]" for a description of the XML struct send. 
+
+\li 2. Status-Messages send from the FirstMix to the InfoService every minute to update the current
+status of the MixCascade including Number of Users, Traffic situation etc.
+This is a POST request and the command name is: "status".
+See \ref XMLMixCascadeStatus "[XML]" for a description of the XML struct send. 
 
-For Upstream and Downstream different keys are used.
 */
 
-
 int main(int argc, const char* argv[])
 	{		
+		/*	UINT64 h=18446744073709551615;
+	//	UINT64 h=184004600;
+		UINT8 bg[255];
+		UINT64 t1,t2;
+		getcurrentTimeMillis(t1);
+		//for(UINT32 i=0;i<10000000;i++)
+			print64(bg,h);
+		getcurrentTimeMillis(t2);
+		printf("%u\n",diff64(t2,t1));
+		printf("%s\n",bg);
+		*/
+		/*	CAQueue oQ;
+		UINT32 l,b;
+		l=4;
+		SINT32 r=oQ.getOrWait((UINT8*)&b,&l,5000);
+		oQ.add(&b,4);
+		r=oQ.getOrWait((UINT8*)&b,&l,5000);
+		r=oQ.getOrWait((UINT8*)&b,&l,5000);
+		*/
+//		UINT16 timestamp;
+//		currentTimestamp((UINT8*)&timestamp);
 		pMix=NULL;
 
 		int i;
-		UINT32 start;
 		SINT32 maxFiles;
 		//Setup Routines
 		XMLPlatformUtils::Initialize();	
@@ -324,6 +484,8 @@ Debug(dc::malloc.on());
 #endif
 
 			//some test....
+		checkSizesOfBaseTypes();
+#ifndef NEW_MIX_TYPE
 		if(MIXPACKET_SIZE!=sizeof(MIXPACKET))
 			{
 				CAMsg::printMsg(LOG_CRIT,"MIXPACKET_SIZE [%u] != sizeof(MUXPACKET) [%u] --> maybe a compiler (optimization) problem!\n",MIXPACKET_SIZE,sizeof(MIXPACKET));
@@ -344,12 +506,7 @@ Debug(dc::malloc.on());
 					exit(-1);
 				CAMsg::printMsg(LOG_CRIT,"Hm, The Offsets seams to be ok - so we try to continue - hope that works...\n");
 			}
-		if(sizeof(UINT32)!=4)
-			{
-				CAMsg::printMsg(LOG_CRIT,"sizeof(UINT32) != 4 --> maybe a compiler (optimization) problem!\n");
-				exit(-1);
-			}
-				
+#endif		
 #ifdef LOG_CRIME
 			testTre();
 #endif
@@ -360,7 +517,13 @@ Debug(dc::malloc.on());
 			err=WSAStartup(0x0202,&wsadata);
 		#endif
 		//initalize Random..
-
+/*		CAListenerInterface* p=CAListenerInterface::getInstance(RAW_TCP,(UINT8*)"141.76.46.17",6789);
+		const CASocketAddr const* a=p->getAddr();
+		CASocketAddr* b=(CASocketAddr*)a;
+		((CASocketAddrINet*)a)->setPort(3);
+		delete a;
+		int z=3;
+*/
 		#if _WIN32
 			RAND_screen();
 		#else
@@ -380,6 +543,7 @@ Debug(dc::malloc.on());
 				goto EXIT;
 			}
 			
+
 /*		UINT8 buff1[1024];
 		UINT32 len=1024;
 		oRsa.getPublicKeyAsXML(buff1,&len);
@@ -438,7 +602,7 @@ Debug(dc::malloc.on());
 		//exit(0);
 		//Testing msSleep
 		CAMsg::printMsg(LOG_DEBUG,"Should sleep now for aprox 2 seconds....\n");
-		start=time(NULL);
+		UINT32 start=time(NULL);
 		for(i=0;i<10;i++)
 			msSleep(200);
 		start=time(NULL)-start;
@@ -482,12 +646,13 @@ Debug(dc::malloc.on());
 					if(options.getLogDir(buff,255)==E_SUCCESS)
 						{
 							if(options.getCompressLogs())
-								CAMsg::setOptions(MSG_COMPRESSED_FILE);
+								CAMsg::setLogOptions(MSG_COMPRESSED_FILE);
 							else
-								CAMsg::setOptions(MSG_FILE);
+								CAMsg::setLogOptions(MSG_FILE);
 						}
 					else
-						CAMsg::setOptions(MSG_LOG);
+						CAMsg::setLogOptions(MSG_LOG);
+					CAMsg::openSpecialLog();
 					pid_t pid;
 					pid=fork();
 					if(pid!=0)
@@ -504,14 +669,14 @@ Debug(dc::malloc.on());
 				if(options.getLogDir((UINT8*)buff,255)==E_SUCCESS)
 					{
 						if(options.getCompressLogs())
-							CAMsg::setOptions(MSG_COMPRESSED_FILE);
+							CAMsg::setLogOptions(MSG_COMPRESSED_FILE);
 						else
-							CAMsg::setOptions(MSG_FILE);
+							CAMsg::setLogOptions(MSG_FILE);
 					}
+				CAMsg::openSpecialLog();
 			}
 		CAMsg::printMsg(LOG_INFO,"Anon proxy started!\n");
-		CAMsg::printMsg(LOG_INFO,"Version: %s\n",MIX_VERSION);
-	  CAMsg::printMsg(LOG_INFO,"Using: %s!\n",OPENSSL_VERSION_TEXT);
+		CAMsg::printMsg(LOG_INFO,MIX_VERSION_INFO);
 #ifdef _DEBUG
 		sockets=0;
 #endif
@@ -530,7 +695,12 @@ Debug(dc::malloc.on());
 //		CARoundTripTime* pRTT=NULL;
 		if(options.isLocalProxy())
 			{
-				pMix=new CALocalProxy();
+				#ifndef NEW_MIX_TYPE
+					pMix=new CALocalProxy();
+				#else
+					CAMsg::printMsg(LOG_CRIT,"Compiled without LocalProxy support!\n");
+					goto EXIT;
+				#endif	
 			}
 		else
 			{
@@ -545,7 +715,11 @@ Debug(dc::malloc.on());
 				if(options.isFirstMix())
 					{
 						CAMsg::printMsg(LOG_INFO,"I am the First MIX..\n");
-						pMix=new CAFirstMix();
+						#if !defined(NEW_MIX_TYPE)
+							pMix=new CAFirstMix();
+						#else
+							pMix=new CAFirstMixNewProtocol();
+						#endif
 					}
 				else if(options.isMiddleMix())
 					{
@@ -553,7 +727,11 @@ Debug(dc::malloc.on());
 						pMix=new CAMiddleMix();
 					}
 				else
-					pMix=new CALastMix();
+						#if !defined(NEW_MIX_TYPE)
+							pMix=new CALastMix();
+						#else
+							pMix=new CALastMixNewProtocol();
+						#endif
 			}
 	  CAMsg::printMsg(LOG_INFO,"Starting MIX...\n");
 		if(pMix->start()!=E_SUCCESS)
